#ifndef _TUTILSVAR_H_
#define _TUTILSVAR_H_

#include <stdio.h>			// vsprintf
#include <string>				// Clase string
#include <cstdarg>			// va_start and this
#include <ctime> 	      	// Needed for time data structures and functions
#include <cmath>				// Mathematical library for M_PI
#include <sstream>			// ostringstream, istringstream
#include <iomanip>			// adjust parameters in cout, etc


//namespace util
class util
{
	public:
	
	/**
	 * sprintf like function for string class
	 */
	static std::string format( const char* fmt, ... )
	{
		 char buf[1 << 16];
		 va_list args;
		 va_start( args, fmt );
		 vsprintf( buf, fmt, args );
		 return std::string(buf);
	}

	/**
	 * Left Trim
	 * @param strin
	 * @return input string modified
	 */
	static std::string & LTrim(std::string &strin)
	{
		int size = strin.size();
		if (size != 0)
		{
			int begin = 0;
			int end = size;
			while ( (strin[begin] == ' ' || strin[begin] == '\t') && (begin < end)) /// Looks for spaces from the beggining of the string. if there is none, copy all string
				begin++;
			if (begin != 0)
				strin = strin.substr(begin,end);
		}
		return(strin);
	}

	/**
	 * Right Trim
	 * @param strin
	 * @return input string modified
	 */
	static std::string &RTrim(std::string &strin)
	{
		unsigned int size = strin.size();
		if (size != 0)
		{
			unsigned int begin = 0;
			unsigned int end = size-1;
			while ( (strin[end] == ' ' || strin[end] == '\t') && (end >= begin)) /// Looks for spaces from the beggining of the string. if there is none, copy all string
				end--;
			if ( end != (size-1) )
				strin = strin.substr(begin,end+1);
		}
		return(strin);
	}

	/**
	 * Right and Left Trim
	 * @param strin
	 * @return input string modified
	 */
	static std::string &RLTrim(std::string &strin) { return(LTrim(RTrim(strin))); };
	
	/**
	 * Convert string to upper case
	 * @param strToConvert
	 * @return string converted
	 */
	static std::string strToUpper(std::string strToConvert)
	{///change each element of the string to upper case
		for(unsigned int i=0;i<strToConvert.length();i++)
		{
			strToConvert[i] = toupper(strToConvert[i]);
		}
		return strToConvert;///return the converted string
	}

	/**
	 * Convert string to lower case
	 * @param strToConvert
	 * @return string converted
	 */
	static std::string strToLower(std::string strToConvert)
	{///change each element of the string to lower case
		for(unsigned int i=0;i<strToConvert.length();i++)
		{
			strToConvert[i] = tolower(strToConvert[i]);
		}
		return strToConvert;///return the converted string
	}

	/**
	 * Convert string to upper case the first letter and lower case the rest
	 * @param strToConvert
	 * @return string converted
	 */
	static std::string strToUpperFirst(std::string strToConvert)
	{///change each element of the string to upper case
		if (strToConvert.length() >= 1)
			strToConvert[0] = toupper(strToConvert[0]);
		for(unsigned int i=1;i<strToConvert.length();i++)
		{
			strToConvert[i] = tolower(strToConvert[i]);
		}
		return strToConvert;///return the converted string
	}

	/**
	 * Return date in format YYMMDD
	 */
	static std::string getDate()
	{
		time_t timer = time(NULL);				/// Get time of day
		struct tm *tblock = localtime(&timer);	/// Converts date/time to a structure
		///__PEUDATE__ = util::format("%d%d%d%d", tblock->tm_isdst, tblock->tm_year, tblock->tm_mon, tblock->tm_mday);
		return ( format("%02d%02d%02d", tblock->tm_year-100, tblock->tm_mon+1, tblock->tm_mday) );
	}

	/**
	 * Return date in format HHMMSS
	 */
	static std::string getTime()
	{
		time_t timer = time(NULL);				/// Get time of day
		struct tm *tblock = localtime(&timer);	/// Converts date/time to a structure
		return ( format("%02d%02d%02d", tblock->tm_hour, tblock->tm_min, tblock->tm_sec) );
	}

	static inline const double& max(const double& a, const double& b) { return (a > b ? a : b); }
	static inline const double& min(const double& a, const double& b) { return (a < b ? a : b); }
	static inline const double& sat(const double& _min, const double& num, const double& _max) { return max(_min, min(num, _max)); }

	static inline const int& max(const int& a, const int& b) { return (a > b ? a : b); }
	static inline const int& min(const int& a, const int& b) { return (a < b ? a : b); }
	static inline const int& sat(const int& _min, const int& num, const int& _max) { return max(_min, min(num, _max)); }
	static inline const int radNormalization(const int& ang) { int tmp = (ang>M_PI) ?(ang-M_PI-M_PI):ang;
																					tmp = (tmp<-M_PI)?(tmp+M_PI+M_PI):tmp;
																					return (tmp); }

	static inline const long double deg2rad(long double a) { a *= M_PI / 180.0f; return (a); }
	static inline const double deg2rad(double a) { a *= M_PI / 180.0f; return (a); }
	static inline const float deg2rad(float a) { a *= M_PI / 180.0f; return (a); }
	static inline const long double rad2deg(long double a) { a *= 180.0f / M_PI; return (a); }
	static inline const double rad2deg(double a) { a *= 180.0f / M_PI; return (a); }
	static inline const float rad2deg(float a) { a *= 180.0f / M_PI; return (a); }

	/*
	// return degrees from rad
	static float deg2rad(float grad)
	{
		return (grad*M_PI/180.0f);
	}

	// return rad from degrees
	static float rad2deg(float rad)
	{
		return (rad*180.0f/M_PI);
	}
	*/
};

/*
 #ifndef DEG2RAD
	#define DEG2RAD M_PI / 180.0f
#endif
#ifndef RAD2DEG
	#define RAD2DEG 180.0f / M_PI
#endif
 */

#endif
