#ifndef _MENUTREE
#define _MENUTREE

/* ************************************************************************** */
/* Includes definitions                                                       */
/* ************************************************************************** */
#include <string>					// string class
#include <vector>					// vector class

class MenuTree
{
	class MenuInt
	{
		public:
			std::string menuName;
			std::string info;
			std::vector<std::string> submenu;
			std::vector<std::string> submenuInfo;
			
			MenuInt(std::string name, std::string i, int s)
			{
				menuName = name;
				info = i;
				submenu.clear();
				submenuInfo.clear();
			}
	};
	
private:
	std::vector < MenuInt > menuint;	/** Internal menu structure */
	int highlightMenu;					/** <10 main menu, > 10 submenu. units are submenu index and decens are main menu index */
	
public:
	
	MenuTree()
	{
		menuint.clear();
		highlightMenu = 0;	/** No menu highlighted by default */
	}
	
	void addMenu(std::string m, std::string i="", int s = false)
	{
		menuint.push_back( MenuInt(m, i, s) );
		highlightMenu = 0;	/** highlights first menu */
	}
	
	void addSubMenu(std::string m, std::string i="")
	{
		if (menuint.size() != 0)
		{
			menuint[menuint.size()-1].submenu.push_back(m);
			menuint[menuint.size()-1].submenuInfo.push_back(i);
		}
	}
	
	std::vector < std::string > getMenu()
	{
		std::vector < std::string > tmp; tmp.clear();
		if ( highlightMenu < 10 )
		{
			for (unsigned int j = 0; j < menuint.size(); j++)
				tmp.push_back(menuint[j].menuName);
		}
		else
		{
			int mainmenuindex = highlightMenu / 10 -1;
			for (unsigned int j = 0; j < menuint[mainmenuindex].submenu.size(); j++)
				tmp.push_back(menuint[mainmenuindex].submenu[j]);
		}
		return (tmp);
	}
	
	int getHighlightedIndex() { return(highlightMenu % 10); };
	
	std::string getHighlightedMenu()
	{
		std::string tmp; tmp.clear();
		std::vector < std::string > tmpMenu = getMenu();
		if ( (highlightMenu % 10) < (int)(tmpMenu.size()) )
			tmp = tmpMenu[highlightMenu % 10];
		return(tmp); 
	};
	
	void nextMenu()
	{
		if ( highlightMenu < 10  )
		{
			highlightMenu++;
			if ( highlightMenu >= (int)(menuint.size()) )
				highlightMenu = 0;
		}
		else
		{
			int rootmenu = highlightMenu / 10;
			int submenu = highlightMenu % 10 + 1;
			if ( submenu >= (int)(menuint[rootmenu-1].submenu.size()) )
				submenu = 0;
			highlightMenu = rootmenu * 10 + submenu;
		}
	}
	
	void previousMenu()
	{
		if ( highlightMenu < 10  )
		{
			highlightMenu--;
			if ( highlightMenu <= -1 )
				highlightMenu = menuint.size() -1;
		}
		else
		{
			int rootmenu = highlightMenu / 10;
			int submenu = (highlightMenu % 10) - 1;
			if ( submenu <= -1 )
				submenu = menuint[rootmenu-1].submenu.size() -1;
			highlightMenu = rootmenu * 10 + submenu;
		}
	}
	
	void enterSubmenu()
	{
		if ( highlightMenu < 10 && menuint[highlightMenu].submenu.size() != 0) 
			highlightMenu = (highlightMenu+1)*10;
	}
	
	void exitSubmenu()
	{
		if ( highlightMenu >= 10) 
			highlightMenu = highlightMenu/10 - 1;
	}
	
	int highlightedIndexHasSubmenu()
	{
		int hasSubmenu = false;
		if ( highlightMenu < 10 && menuint[highlightMenu].submenu.size() != 0) 
			hasSubmenu = true;
		return (hasSubmenu);
	}
	
	std::string getHighlightedMenuInfo()
	{
		std::string tmp; tmp.clear();
		if ( highlightMenu < 10 )
		{
			if (highlightMenu < (int)(menuint.size()) )
				tmp = menuint[highlightMenu].info;
		}
		else
		{
			int mainmenuindex = highlightMenu / 10 -1;
			tmp = menuint[mainmenuindex].submenuInfo[highlightMenu%10];
		}
		return (tmp);
	};
	
	void setHighlightedMenuInfo(std::string i)
	{
		if ( highlightMenu < 10 )
			menuint[highlightMenu].info = i;
		else
		{
			int mainmenuindex = highlightMenu / 10 -1;
			menuint[mainmenuindex].submenuInfo[highlightMenu%10] = i;
		}
	};
	
	
};

#endif
