#ifndef _MENUTREEGUI
#define _MENUTREEGUI

/* ************************************************************************** */
/* Includes definitions                                                       */
/* ************************************************************************** */
#include <cstdio>					// fgets, fopen, perror, printf, sprintf
#include <string>					// string class
#include <vector>					// vector class
#include "./menuTree.h"			// Utils library (uses util::format)
#include <cv.h>					// includes OpenCV definitions
#include <highgui.h>				// includes highGUI definitions

class MenuTreeGUI : public MenuTree
{
	
private:
	int windowWidth;			// window width
	int windowHeight;			// window height
	int menuLeftWidth;		// left menu width
	int menuBottomHeight;	// bottom menu height
	int lineWidth;				// width of line to draw
	int borderWidth;			// empty button border
	
	
public:
	
	MenuTreeGUI() : MenuTree(), windowWidth(355), windowHeight(340), menuLeftWidth(100), menuBottomHeight(35), lineWidth(1), borderWidth(1)
	{
	};

	void setMenuDim(int ww, int wh, int mlw = 100, int mbh = 35, int lw = 1, int bw = 2)
	{
		windowWidth = ww;
		windowHeight = wh;
		menuLeftWidth = mlw - lw;
		menuBottomHeight = mbh;
		lineWidth = lw;
		borderWidth = bw;
	}


	void drawLeftMenu(cv::Mat &img)
	{
		if (img.rows < windowHeight || img.cols < windowWidth)
			return;
		cv::rectangle(img, cv::Point(0,0), cv::Point(menuLeftWidth, windowHeight), cv::Scalar(0,0,0), -1, 8, 0);
		cv::Scalar menuColor(104,104,104);
		cv::Scalar selectedColor = cv::Scalar(204,204,204);
		cv::line(img, cv::Point(menuLeftWidth, 0), cv::Point(menuLeftWidth, windowHeight), selectedColor, lineWidth);
		std::vector < std::string > m = getMenu();
		int nMenus = m.size();
		int hli = getHighlightedIndex();
		if (nMenus > 0)
		{
			int menuButtonHeight = (windowHeight - borderWidth)/nMenus;
			for(int i = 0; i < nMenus; i++)
			{	// Draw buttons borders
				cv::Scalar color = menuColor;
				if ( i == hli)
					color = selectedColor;
				cv::Point Pto1(borderWidth, borderWidth + menuButtonHeight*i);
				cv::Point Pto2(menuLeftWidth - borderWidth*2, (i+1)*menuButtonHeight - borderWidth);
				cv::rectangle(img, Pto1, Pto2, color, 1, 8, 0);
				writeText(img, m[i], cv::Point(5, (Pto1.y+Pto2.y)/2), color);
			}
		}
	}
	
	void drawBottomMenu(cv::Mat &img, std::vector <std::string > text)
	{
		cv::rectangle(img, cv::Point(menuLeftWidth + lineWidth, windowHeight - menuBottomHeight), cv::Point(windowWidth, windowHeight), cv::Scalar(0,0,0), -1, 8, 0);
		cv::Scalar menuColor(204,204,204);
		cv::line(img, cv::Point(menuLeftWidth, windowHeight - menuBottomHeight), cv::Point(windowWidth, windowHeight - menuBottomHeight), cv::Scalar(204,204,204), lineWidth);
		int textLineOffset = 14;
		for (unsigned int i = 0; i <text.size(); i++)
			writeText(img, text[i], cv::Point(menuLeftWidth+3, windowHeight - menuBottomHeight -2 + textLineOffset*(i+1)), menuColor);
	}
	
	void drawBottomMenu(cv::Mat &img)
	{
		cv::rectangle(img, cv::Point(menuLeftWidth + lineWidth, windowHeight - menuBottomHeight), cv::Point(windowWidth, windowHeight), cv::Scalar(0,0,0), -1, 8, 0);
		cv::Scalar menuColor(204,204,204);
		cv::line(img, cv::Point(menuLeftWidth, windowHeight - menuBottomHeight), cv::Point(windowWidth, windowHeight - menuBottomHeight), cv::Scalar(204,204,204), lineWidth);
		int textLineOffset = 14;
		writeText(img, getHighlightedMenu() + ":", cv::Point(menuLeftWidth+3, windowHeight - menuBottomHeight -2 + textLineOffset*(0+1)), menuColor);
		writeText(img, getHighlightedMenuInfo(), cv::Point(menuLeftWidth+3, windowHeight - menuBottomHeight -2 + textLineOffset*(1+1)), menuColor);
	}
	
	void writeText(cv::Mat &img, std::string str, cv::Point pto, cv::Scalar color = cv::Scalar(204,204,204))
	{
		float fontSize = 0.4f;
		cv::putText(img, str, pto, CV_FONT_HERSHEY_SIMPLEX, fontSize, color);
	}
	
};

#endif
