/* ************************************************************************** */
/* Includes definitions                                                       */
/* ************************************************************************** */
#include "opencvBasics.h"

CamMenu::CamMenu()
{
	imgSize = cv::Size(320, 240);
	menuImg = cv::Mat(imgSize.height + _MENUBOTTOMHEIGHT, imgSize.width + _MENULEFTWIDTH, CV_8UC3, cv::Scalar(0, 0, 0));
	menutreegui.setMenuDim(imgSize.width + _MENULEFTWIDTH, imgSize.height + _MENUBOTTOMHEIGHT, _MENULEFTWIDTH, _MENUBOTTOMHEIGHT);
	showImg = cv::Mat(imgSize.height, imgSize.width, CV_8UC3, cv::Scalar(155));
	key = 0;
	salir = false;
	operacion.clear();
	streamCapture.clear();
	streamWriter.clear();
	initMenu();
}

int CamMenu::Exit()
{
	return (salir);
}

void CamMenu::initMenu()
{
	menutreegui.addMenu("Informacion", "Use los cursores/intro para navegar");
	menutreegui.addMenu("Input/Output", "Lectura/escritura de imagenes/camara...");
		menutreegui.addSubMenu("Leer imagen", "Lee una imagen del disco duro");
		menutreegui.addSubMenu("Guardar imagen", "Guarda una imagen en save.jpg");
		menutreegui.addSubMenu("Abrir camara", "Abre la camara conectada al PC");
		menutreegui.addSubMenu("Abrir video", "Abre un video del disco duro");
		menutreegui.addSubMenu("Guardar video", "Guarda imagen en save.avi");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Graficos", "Graficos basicos: texto, lineas, circulos...");
	menutreegui.addMenu("Operaciones", "Operaciones basicas con matrices e imagenes");
		menutreegui.addSubMenu("Suma matrices", "Suma a imgFigGeom.png una imagen verde");
		menutreegui.addSubMenu("Resta matrices", "Resta a imgFigGeom.png una imagen verde");
		menutreegui.addSubMenu("Suma escalar", "Suma a imgFigGeom.png una valor rojo a 155");
		menutreegui.addSubMenu("Invertir imagen", "Invierte verticalmente imgFigGeom.png");
		menutreegui.addSubMenu("Escalar", "Reescala imgFigGeom.png a 320x240 pixeles");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Trans. Color", "Transformaciones del espacio de color");
		menutreegui.addSubMenu("RGB", "Muestra imgFigGeom.png en RGB");
		menutreegui.addSubMenu("HSV", "Muestra imgFigGeom.png en HSV");
		menutreegui.addSubMenu("H", "Muestra el canal H de imgFigGeom.png");
		menutreegui.addSubMenu("S", "Muestra el canal S de imgFigGeom.png");
		menutreegui.addSubMenu("V", "Muestra el canal V de imgFigGeom.png");
		menutreegui.addSubMenu("Umbralizacion", "Aplica funcion con umbral 125 a imgFigGeom.png");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Algoritmos", "Algoritmos avanzados para deteccion");
		menutreegui.addSubMenu("Detectar bordes", "Aplica deteccion de bordes a imgFigGeom.png");
		menutreegui.addSubMenu("Detectar lineas", "Aplica deteccion de lineas a imgFigGeom.png");
		menutreegui.addSubMenu("Detectar caras", "Aplica deteccion de caras a imgCello.jpg");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
	menutreegui.addMenu("Salir", "Salir de la aplicacion");
		menutreegui.addSubMenu("Salir", "Salir de la aplicacion");
		menutreegui.addSubMenu("Volver", "Vuelve al menu anterior");
}

int CamMenu::update(int wait)
{
	leeEntradaDeUsuario();
	menutreegui.drawLeftMenu(menuImg);
	menutreegui.drawBottomMenu(menuImg);
	ejecutarOperacion();
	cv::Mat tmpImg = cv::Mat(menuImg, cv::Rect( _MENULEFTWIDTH, 0, imgSize.width, imgSize.height) );
	showImg.copyTo(tmpImg);
	cv::imshow("Funcionalidad de OpenCV", menuImg);
	key = cv::waitKey(wait);
	//if (key == 's')
	//	cv::imwrite("save.jpg", showImg);	
	return (key);
}

void CamMenu::ejecutarOperacion()
{
	showImg.setTo( cv::Scalar(155,0,0) );
	menuInputOutput();
	menuGraficos();
	menuOperacionesBasicas();
	menuTransformacionesDeColor();
	menuAlgoritmosAvanzados();
}

void CamMenu::menuInputOutput()
{
	if ( operacion.compare("Leer imagen") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgCello.jpg");
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
	}
	if ( operacion.compare("Guardar imagen") == 0 )
	{	
		cv::putText(showImg, "Imagen guardada con OpenCV", cv::Point(15, 100), CV_FONT_HERSHEY_SIMPLEX, 0.55, cv::Scalar(255,255,0));
		cv::imwrite("save.jpg", showImg);
	}
	if ( operacion.compare("Abrir camara") == 0 )
	{	// Si no hay camara abierta la abre. Si hay una abierta la cierra
		if ( streamCapture.empty() )
			streamCapture.push_back( cv::VideoCapture(0) );
		else
			streamCapture.clear();
		operacion = "";
	}
	if ( operacion.compare("Abrir video") == 0 )
	{	// Si no hay video abierto lo abre. Si hay video abierto lo cierra
		if ( streamCapture.empty() )
			streamCapture.push_back( cv::VideoCapture("video.avi") );
		else
			streamCapture.clear();
		operacion = "";
	}
	if ( operacion.compare("Guardar video") == 0 )
	{	// Si no hay video abierto lo abre. Si esta abierto lo cierra.
		if ( streamWriter.empty() )
			streamWriter.push_back( cv::VideoWriter( "save.avi", CV_FOURCC('x','v','i','d'), 25, imgSize ,1) );
		else
			streamWriter.clear();
		operacion.clear();
	}

	// Si la camara esta abierta lee un fotograma
	if ( !streamCapture.empty() )
	{
		cv::Mat tmpImg;
		streamCapture[0] >> tmpImg;
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
	}
	// Si hay un flujo de video de salida abierto, se guardan las imagenes que se muestran en pantalla.
	if ( !streamWriter.empty() )
	{
		streamWriter[0] << showImg;
	}
}

void CamMenu::menuGraficos()
{
	if ( operacion.compare("Graficos") == 0 )
	{
		cv::rectangle(showImg, cv::Point(5,15), cv::Point(250, 150), cv::Scalar(0,0,255), -1, CV_AA, 0);
		cv::rectangle(showImg, cv::Point(105,150), cv::Point(300, 220), cv::Scalar(0,255,255), 5, CV_AA, 0);
		cv::circle(showImg, cv::Point(50, 200), 20, cv::Scalar(255,0,255), -1, CV_AA, 0);
		cv::circle(showImg, cv::Point(125, 200), 40, cv::Scalar(55,155,55), 3, CV_AA, 0);
		cv::line(showImg, cv::Point(60, 150), cv::Point(285, 55), cv::Scalar(105,155,5), 2);
		cv::line(showImg, cv::Point(160, 50), cv::Point(85, 155), cv::Scalar(205,55,95), 7);
		cv::putText(showImg, "Texto con OpenCV", cv::Point(15, 20), CV_FONT_HERSHEY_SIMPLEX, 0.55, cv::Scalar(255,255,0));
		cv::putText(showImg, "Texto con OpenCV", cv::Point(25, 100), CV_FONT_HERSHEY_SIMPLEX, 0.75, cv::Scalar(55,255,255));
	}
}

void CamMenu::menuOperacionesBasicas()
{
	if ( operacion.compare("Suma matrices") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png");
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
		cv::Mat img2 = cv::Mat(imgSize.height, imgSize.width, CV_8UC3, cv::Scalar(0,255,0));
		showImg += img2;
	}
	if ( operacion.compare("Resta matrices") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png");
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
		cv::Mat img2 = cv::Mat(imgSize.height, imgSize.width, CV_8UC3, cv::Scalar(0,255,0));
		showImg -= img2;
	}
	if ( operacion.compare("Suma escalar") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png");
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
		showImg += cv::Scalar(0, 0, 155);
	}
	if ( operacion.compare("Invertir imagen") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png");
		if ( !tmpImg.empty() )
		{
			cv::flip(tmpImg, tmpImg, 0);
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
		}
	}
	if ( operacion.compare("Escalar") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png");
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
	}		
}

void CamMenu::menuTransformacionesDeColor()
{
	if ( operacion.compare("RGB") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png", 1);
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
	}
	if ( operacion.compare("HSV") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png", 1);
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
		cv::cvtColor( showImg, showImg, CV_BGR2HSV );
	}
	if ( operacion.compare("H") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png", 1);
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
		std::vector<cv::Mat> splitted;
		cv::cvtColor( showImg, showImg, CV_BGR2HSV );
		cv::split( showImg, splitted );
		splitted[1] = splitted[0]; splitted[2] = splitted[0];
		cv::merge( splitted, showImg);
	}
	if ( operacion.compare("S") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png", 1);
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
		std::vector<cv::Mat> splitted;
		cv::cvtColor( showImg, showImg, CV_BGR2HSV );
		cv::split( showImg, splitted );
		splitted[0] = splitted[1]; splitted[2] = splitted[1];
		cv::merge( splitted, showImg);
	}
	if ( operacion.compare("V") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png", 1);
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
		std::vector<cv::Mat> splitted;
		cv::cvtColor( showImg, showImg, CV_BGR2HSV );
		cv::split( showImg, splitted );
		splitted[0] = splitted[2]; splitted[1] = splitted[2];
		cv::merge( splitted, showImg);
	}
	if ( operacion.compare("Umbralizacion") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png", 0);
		cv::Mat showImgGray = cv::Mat(imgSize.height, imgSize.width, CV_8UC1, cv::Scalar(0));
		if ( !tmpImg.empty() )
			cv::resize( tmpImg, showImgGray , showImgGray.size(), 0, 0, cv::INTER_LINEAR );
		cv::threshold(showImgGray, showImgGray, 125, 255, cv::THRESH_BINARY);
		cv::cvtColor( showImgGray, showImg, CV_GRAY2BGR );
	}
}

void CamMenu::menuAlgoritmosAvanzados()
{
	if ( operacion.compare("Detectar bordes") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png");
		if ( !tmpImg.empty() )
		{
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
			cv::Mat tmpImgGray;
			cv::cvtColor(showImg, tmpImgGray, CV_BGR2GRAY);
			cv::Canny( tmpImgGray, tmpImgGray, 50, 200, 3 );
			cv::cvtColor(tmpImgGray, showImg, CV_GRAY2BGR);
		}
	}
	if ( operacion.compare("Detectar lineas") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgFigGeom.png");
		if ( !tmpImg.empty() )
		{
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
			cv::Mat tmpImgGray;
			cv::cvtColor(showImg, tmpImgGray, CV_BGR2GRAY);
			std::vector<cv::Vec4i> lines;
			cv::Canny( tmpImgGray, tmpImgGray, 50, 200, 3 );
			cv::HoughLinesP( tmpImgGray, lines, 1, CV_PI/180, 80, 30, 10 );
			for( size_t i = 0; i < lines.size(); i++ )
				cv::line( showImg, cv::Point(lines[i][0], lines[i][1]), cv::Point(lines[i][2], lines[i][3]), cv::Scalar(0,0,255), 3, 8 );
		}
	}
	if ( operacion.compare("Detectar caras") == 0 )
	{
		cv::Mat tmpImg = cv::imread("imgCello.jpg");
		std::string cascadeName = "haarcascades/haarcascade_frontalface_default.xml";
		cv::CascadeClassifier cascade;				/** haar cascade classifier */
		std::vector<cv::Rect> facesList;				/** list of faces */
		cv::Size minFaceSize(15, 15);					/** Tamaño minimo de caras a deteactar */
		if ( !tmpImg.empty() && cascade.load( cascadeName ))
		{
			cv::resize( tmpImg, showImg , showImg.size(), 0, 0, cv::INTER_LINEAR );
			if ( !cascade.empty() )
			{
				cv::Mat grayImage = cv::Mat(imgSize.height, imgSize.width, CV_8UC1, cv::Scalar(0));
				cv::cvtColor( showImg, grayImage, CV_BGR2GRAY );
				cascade.detectMultiScale( grayImage , facesList, 1.1, 2, CV_HAAR_DO_CANNY_PRUNING, minFaceSize );
				for( unsigned int i = 0; i < facesList.size(); i++ )
					cv::rectangle(showImg, cv::Point(facesList[i].x, facesList[i].y), cv::Point(facesList[i].x + facesList[i].width, facesList[i].y + facesList[i].height), cv::Scalar(0,255,0), 2, CV_AA, 0);
			}
		}
	}
}

void CamMenu::leeEntradaDeUsuario()
{
	if ( key == 82)
		menutreegui.previousMenu();
	else if ( key == 84)
		menutreegui.nextMenu();
	else if ( key == 27 )
	{
		menutreegui.exitSubmenu();
	}
	else if ( key == 10)
	{
		std::string mSelected = menutreegui.getHighlightedMenu();
		if ( menutreegui.highlightedIndexHasSubmenu() )
			menutreegui.enterSubmenu();
		else if ( mSelected.compare("Volver") == 0 )
		{
			menutreegui.exitSubmenu();
			operacion.clear();
			streamWriter.clear();
			streamCapture.clear();
		}
		else if ( mSelected.compare("Salir") == 0 )
			salir = true;
		else
			operacion = mSelected;
	}
}
