#ifndef _CAMMENU
#define _CAMMENU

/* ************************************************************************** */
/* Includes definitions                                                       */
/* ************************************************************************** */
#define __mdebug printf("%s:%d\n", __FILE__, __LINE__)
#include <cstdio>					// fgets, fopen, perror, printf, sprintf
#include <string>					// string class
#include <vector>					// vector class
#include "./libUtilsVar.h"		// Utils library (uses util::format)
#include <cv.h>					// includes OpenCV definitions
#include <highgui.h>				// includes highGUI definitions
#include "menuTreeGUI.h"		// class that represents graphically menu tree


		
class CamMenu
{
private:
	cv::Size imgSize;										/** Tamaño de la imagen a mostrar */
	static const int _MENULEFTWIDTH = 115;			/** Tamaño por defecto del menu lateral */
	static const int _MENUBOTTOMHEIGHT = 35;		/** Tamaño por defecto de menu inferior */
	std::vector <cv::VideoCapture> streamCapture;/** flujo de captura de la camara */
	std::vector <cv::VideoWriter> streamWriter;	/** flujo de salida de video */
	cv::Mat menuImg;										/** imagen que contiene la imagen mostrada y el menu de usuario */
	cv::Mat showImg;										/** imagen mostrada */
	MenuTreeGUI menutreegui;							/** Clase que muestra un menu grafico al usuario */
	char key;												/** tecla pulsada por el usuario */
	int salir;												/** true cuando se pulse salir en el menu */
	std::string operacion;								/** Indica la operacion a realizar sobre la imagen */
	
public:	
	CamMenu();

	/**
	 * Actualiza el interfaz grafico
	 */
	int update(int wait = 50);
	
	/**
	 * Devuelve verdadero si se ha seleccionado salir en el menu
	 */
	int Exit();
	
private:
	
	/**
	 * Inicializa el menu de usuario
	 */
	void initMenu();
	
	/**
	 * Llama a los submenus para realizar operaciones sobre la imagen segun la eleccion del usuario
	 */
	void ejecutarOperacion();
	
	/**
	 * Ejecuta las operaciones correspondientes al menu de input output
	 */
	void menuInputOutput();
	
	/**
	 * Ejecuta las operaciones correspondientes al menu de graficos
	 */
	void menuGraficos();
	
	/**
	 * Ejecuta las operaciones correspondientes al menu de operaciones basicas
	 */
	void menuOperacionesBasicas();
	
	/**
	 * Ejecuta las operaciones correspondientes al menu de transformaciones de color
	 */
	void menuTransformacionesDeColor();
	
	/**
	 * Ejecuta las operaciones correspondientes al menu de algoritmos avanzados
	 */
	void menuAlgoritmosAvanzados();
		
	/**
	 * Lee la entrada de menu seleccionada por el usuario
	 */
	void leeEntradaDeUsuario();
};

#endif
